@@ -33,12 +33,12 @@
 # ----------------------------------------------------------------------------
 # $Id:$
 
-'''Formatted and unformatted document interfaces used by text layout.
+"""Formatted and unformatted document interfaces used by text layout.
 
 Abstract representation
 =======================
 
-Styled text in pyglet is represented by one of the `AbstractDocument` classes,
+Styled text in pyglet is represented by one of the :py:class:`~pyglet.text.document.AbstractDocument` classes,
 which manage the state representation of text and style independently of how
 it is loaded or rendered.  
 
@@ -67,7 +67,7 @@ This can be useful to tag areas of interest in a document, or maintain
 references back to the source material.
 
 As well as text, the document can contain arbitrary elements represented by
-`InlineElement`.  An inline element behaves like a single character in the
+:py:class:`~pyglet.text.document.InlineElement`.  An inline element behaves like a single character in the
 documented, but can be rendered by the application.
 
 Paragraph breaks
@@ -84,16 +84,16 @@ http://unicode.org/reports/tr13/tr13-5.html.
 Document classes
 ================
 
-Any class implementing `AbstractDocument` provides an interface to a
+Any class implementing :py:class:`~pyglet.text.document.AbstractDocument` provides an interface to a
 document model as described above.  In theory a structured document such as
 HTML or XML could export this model, though the classes provided by pyglet
 implement only unstructured documents.
 
-The `UnformattedDocument` class assumes any styles set are set over the entire
+The :py:class:`~pyglet.text.document.UnformattedDocument` class assumes any styles set are set over the entire
 document.  So, regardless of the range specified when setting a ``"bold"``
 style attribute, for example, the entire document will receive that style.
 
-The `FormattedDocument` class implements the document model directly, using
+The :py:class:`~pyglet.text.document.FormattedDocument` class implements the document model directly, using
 the `RunList` class to represent style runs efficiently.
 
 Style attributes
@@ -102,7 +102,7 @@ Style attributes
 The following character style attribute names are recognised by pyglet:
 
 ``font_name``
-    Font family name, as given to `pyglet.font.load`.
+    Font family name, as given to :py:func:`pyglet.font.load`.
 ``font_size``
     Font size, in points.
 ``bold``
@@ -161,8 +161,10 @@ All style attributes (including those not present in a document) default to
 ``None`` (including the so-called "boolean" styles listed above).  The meaning
 of a ``None`` style is style- and application-dependent. 
 
-:since: pyglet 1.1
-'''
+.. versionadded:: 1.1
+"""
+from builtins import next
+from builtins import object
 
 __docformat__ = 'restructuredtext'
 __version__ = '$Id: $'
@@ -173,13 +175,14 @@ import sys
 from pyglet import event
 from pyglet.text import runlist
 
-_is_epydoc = hasattr(sys, 'is_epydoc') and sys.is_epydoc
+_is_pyglet_docgen = hasattr(sys, 'is_pyglet_docgen') and sys.is_pyglet_docgen
 
 #: The style attribute takes on multiple values in the document.
 STYLE_INDETERMINATE = 'indeterminate'
 
+
 class InlineElement(object):
-    '''Arbitrary inline element positioned within a formatted document.
+    """Arbitrary inline element positioned within a formatted document.
 
     Elements behave like a single glyph in the document.  They are
     measured by their horizontal advance, ascent above the baseline, and
@@ -203,7 +206,8 @@ class InlineElement(object):
         `advance` : int
             Width of the element, in pixels.
 
-    '''
+    """
+
     def __init__(self, ascent, descent, advance):
         self.ascent = ascent
         self.descent = descent
@@ -211,14 +215,14 @@ class InlineElement(object):
         self._position = None
 
     position = property(lambda self: self._position,
-                        doc='''Position of the element within the
+                        doc="""Position of the element within the
         document.  Read-only.
 
         :type: int
-        ''')
+        """)
 
     def place(self, layout, x, y):
-        '''Construct an instance of the element at the given coordinates.
+        """Construct an instance of the element at the given coordinates.
 
         Called when the element's position within a layout changes, either
         due to the initial condition, changes in the document or changes in
@@ -241,11 +245,11 @@ class InlineElement(object):
                 Position of the baseline, relative to the top edge of the
                 document, in pixels.  Note that this is typically negative.
 
-        '''
+        """
         raise NotImplementedError('abstract')
 
     def remove(self, layout):
-        '''Remove this element from a layout.
+        """Remove this element from a layout.
 
         The counterpart of `place`; called when the element is no longer
         visible in the given layout.
@@ -254,58 +258,59 @@ class InlineElement(object):
             `layout` : `pyglet.text.layout.TextLayout`
                 The layout the element was removed from.
 
-        '''
+        """
         raise NotImplementedError('abstract')
 
+
 class AbstractDocument(event.EventDispatcher):
-    '''Abstract document interface used by all `pyglet.text` classes.
+    """Abstract document interface used by all :py:mod:`pyglet.text` classes.
 
     This class can be overridden to interface pyglet with a third-party
     document format.  It may be easier to implement the document format in
-    terms of one of the supplied concrete classes `FormattedDocument` or
-    `UnformattedDocument`. 
-    '''
-    _previous_paragraph_re = re.compile('\n[^\n\u2029]*$')
-    _next_paragraph_re = re.compile('[\n\u2029]')
+    terms of one of the supplied concrete classes :py:class:`~pyglet.text.document.FormattedDocument` or
+    :py:class:`~pyglet.text.document.UnformattedDocument`. 
+    """
+    _previous_paragraph_re = re.compile(u'\n[^\n\u2029]*$')
+    _next_paragraph_re = re.compile(u'[\n\u2029]')
 
     def __init__(self, text=''):
         super(AbstractDocument, self).__init__()
-        self._text = ''
+        self._text = u''
         self._elements = []
         if text:
             self.insert_text(0, text)
 
-    def _get_text(self):
+    @property
+    def text(self):
+        """Document text.
+
+        For efficient incremental updates, use the :py:func:`~pyglet.text.document.AbstractDocument.insert_text` and
+        :py:func:`~pyglet.text.document.AbstractDocument.delete_text` methods instead of replacing this property.
+
+        :type: str
+        """
         return self._text
 
-    def _set_text(self, text):
+    @text.setter
+    def text(self, text):
         if text == self._text:
             return
         self.delete_text(0, len(self._text))
         self.insert_text(0, text)
-    
-    text = property(_get_text, _set_text, 
-                    doc='''Document text.
-                   
-        For efficient incremental updates, use the `insert_text` and
-        `delete_text` methods instead of replacing this property.
-        
-        :type: str
-        ''')
 
     def get_paragraph_start(self, pos):
-        '''Get the starting position of a paragraph.
+        """Get the starting position of a paragraph.
 
         :Parameters:
             `pos` : int
                 Character position within paragraph.
 
         :rtype: int
-        '''
+        """
         # Tricky special case where the $ in pattern matches before the \n at
         # the end of the string instead of the end of the string.
-        if (self._text[:pos + 1].endswith('\n') or 
-            self._text[:pos + 1].endswith('\u2029')):
+        if (self._text[:pos + 1].endswith('\n') or
+                self._text[:pos + 1].endswith(u'\u2029')):
             return pos
 
         m = self._previous_paragraph_re.search(self._text, 0, pos + 1)
@@ -314,32 +319,32 @@ class AbstractDocument(event.EventDispatcher):
         return m.start() + 1
 
     def get_paragraph_end(self, pos):
-        '''Get the end position of a paragraph.
+        """Get the end position of a paragraph.
 
         :Parameters:
             `pos` : int
                 Character position within paragraph.
 
         :rtype: int
-        '''
+        """
         m = self._next_paragraph_re.search(self._text, pos)
         if not m:
             return len(self._text)
         return m.start() + 1
 
     def get_style_runs(self, attribute):
-        '''Get a style iterator over the given style attribute.
+        """Get a style iterator over the given style attribute.
 
         :Parameters:
             `attribute` : str
                 Name of style attribute to query.
 
         :rtype: `AbstractRunIterator`
-        '''
+        """
         raise NotImplementedError('abstract')
 
     def get_style(self, attribute, position=0):
-        '''Get an attribute style at the given position.
+        """Get an attribute style at the given position.
 
         :Parameters:
             `attribute` : str
@@ -348,11 +353,11 @@ class AbstractDocument(event.EventDispatcher):
                 Character position of document to query.
 
         :return: The style set for the attribute at the given position.
-        '''
+        """
         raise NotImplementedError('abstract')
 
     def get_style_range(self, attribute, start, end):
-        '''Get an attribute style over the given range.
+        """Get an attribute style over the given range.
 
         If the style varies over the range, `STYLE_INDETERMINATE` is returned.
 
@@ -366,16 +371,16 @@ class AbstractDocument(event.EventDispatcher):
 
         :return: The style set for the attribute over the given range, or
             `STYLE_INDETERMINATE` if more than one value is set.
-        '''
-        iter = self.get_style_runs(attribute)
-        _, value_end, value = next(iter.ranges(start, end))
+        """
+        iterable = self.get_style_runs(attribute)
+        _, value_end, value = next(iterable.ranges(start, end))
         if value_end < end:
             return STYLE_INDETERMINATE
         else:
             return value
 
     def get_font_runs(self, dpi=None):
-        '''Get a style iterator over the `pyglet.font.Font` instances used in
+        """Get a style iterator over the `pyglet.font.Font` instances used in
         the document.
 
         The font instances are created on-demand by inspection of the
@@ -385,14 +390,14 @@ class AbstractDocument(event.EventDispatcher):
         :Parameters:
             `dpi` : float
                 Optional resolution to construct fonts at.  See
-                `pyglet.font.load`.
+                :py:func:`pyglet.font.load`.
 
         :rtype: `AbstractRunIterator`
-        '''
+        """
         raise NotImplementedError('abstract')
 
     def get_font(self, position, dpi=None):
-        '''Get the font instance used at the given position.
+        """Get the font instance used at the given position.
 
         :see: `get_font_runs`
 
@@ -401,15 +406,15 @@ class AbstractDocument(event.EventDispatcher):
                 Character position of document to query.
             `dpi` : float
                 Optional resolution to construct fonts at.  See
-                `pyglet.font.load`.
+                :py:func:`pyglet.font.load`.
 
         :rtype: `pyglet.font.Font`
         :return: The font at the given position.
-        '''
+        """
         raise NotImplementedError('abstract')
-    
+
     def insert_text(self, start, text, attributes=None):
-        '''Insert text into the document.
+        """Insert text into the document.
 
         :Parameters:
             `start` : int
@@ -420,19 +425,19 @@ class AbstractDocument(event.EventDispatcher):
                 Optional dictionary giving named style attributes of the
                 inserted text.
 
-        '''
+        """
         self._insert_text(start, text, attributes)
         self.dispatch_event('on_insert_text', start, text)
 
     def _insert_text(self, start, text, attributes):
-        self._text = ''.join((self._text[:start], text, self._text[start:]))
+        self._text = u''.join((self._text[:start], text, self._text[start:]))
         len_text = len(text)
         for element in self._elements:
             if element._position >= start:
                 element._position += len_text
 
     def delete_text(self, start, end):
-        '''Delete text from the document.
+        """Delete text from the document.
 
         :Parameters:
             `start` : int
@@ -440,7 +445,7 @@ class AbstractDocument(event.EventDispatcher):
             `end` : int
                 Ending character position to delete to (exclusive).
 
-        '''
+        """
         self._delete_text(start, end)
         self.dispatch_event('on_delete_text', start, end)
 
@@ -448,50 +453,50 @@ class AbstractDocument(event.EventDispatcher):
         for element in list(self._elements):
             if start <= element._position < end:
                 self._elements.remove(element)
-            elif element._position >= end: # fix bug 538
+            elif element._position >= end:  # fix bug 538
                 element._position -= (end - start)
 
         self._text = self._text[:start] + self._text[end:]
 
     def insert_element(self, position, element, attributes=None):
-        '''Insert a element into the document.
+        """Insert a element into the document.
 
-        See the `InlineElement` class documentation for details of
+        See the :py:class:`~pyglet.text.document.InlineElement` class documentation for details of
         usage.
 
         :Parameters:
             `position` : int
                 Character insertion point within document.
-            `element` : `InlineElement`
+            `element` : `~pyglet.text.document.InlineElement`
                 Element to insert.
             `attributes` : dict
                 Optional dictionary giving named style attributes of the
                 inserted text.
 
-        '''
+        """
         assert element._position is None, \
             'Element is already in a document.'
         self.insert_text(position, '\0', attributes)
         element._position = position
         self._elements.append(element)
-        self._elements.sort(key=lambda d:d.position)
+        self._elements.sort(key=lambda d: d.position)
 
     def get_element(self, position):
-        '''Get the element at a specified position.
+        """Get the element at a specified position.
 
         :Parameters:
             `position` : int
                 Position in the document of the element.
 
-        :rtype: `InlineElement`
-        '''
+        :rtype: :py:class:`~pyglet.text.document.InlineElement`
+        """
         for element in self._elements:
             if element._position == position:
                 return element
         raise RuntimeError('No element at position %d' % position)
 
     def set_style(self, start, end, attributes):
-        '''Set text style of some or all of the document.
+        """Set text style of some or all of the document.
 
         :Parameters:
             `start` : int
@@ -501,7 +506,7 @@ class AbstractDocument(event.EventDispatcher):
             `attributes` : dict
                 Dictionary giving named style attributes of the text.
 
-        '''
+        """
         self._set_style(start, end, attributes)
         self.dispatch_event('on_style_text', start, end, attributes)
 
@@ -509,7 +514,7 @@ class AbstractDocument(event.EventDispatcher):
         raise NotImplementedError('abstract')
 
     def set_paragraph_style(self, start, end, attributes):
-        '''Set the style for a range of paragraphs.
+        """Set the style for a range of paragraphs.
 
         This is a convenience method for `set_style` that aligns the
         character range to the enclosing paragraph(s).
@@ -522,15 +527,15 @@ class AbstractDocument(event.EventDispatcher):
             `attributes` : dict
                 Dictionary giving named style attributes of the paragraphs.
 
-        '''
+        """
         start = self.get_paragraph_start(start)
         end = self.get_paragraph_end(end)
         self._set_style(start, end, attributes)
         self.dispatch_event('on_style_text', start, end, attributes)
 
-    if _is_epydoc:
+    if _is_pyglet_docgen:
         def on_insert_text(self, start, text):
-            '''Text was inserted into the document.
+            """Text was inserted into the document.
 
             :Parameters:
                 `start` : int
@@ -539,10 +544,10 @@ class AbstractDocument(event.EventDispatcher):
                     The text that was inserted.
 
             :event:
-            '''
+            """
 
         def on_delete_text(self, start, end):
-            '''Text was deleted from the document.
+            """Text was deleted from the document.
 
             :Parameters:
                 `start` : int
@@ -551,10 +556,10 @@ class AbstractDocument(event.EventDispatcher):
                     Ending character position of deleted text (exclusive).
 
             :event:
-            '''
+            """
 
         def on_style_text(self, start, end, attributes):
-            '''Text character style was modified.
+            """Text character style was modified.
 
             :Parameters:
                 `start` : int
@@ -566,18 +571,21 @@ class AbstractDocument(event.EventDispatcher):
                     text.
 
             :event:
-            '''
+            """
+
+
 AbstractDocument.register_event_type('on_insert_text')
 AbstractDocument.register_event_type('on_delete_text')
 AbstractDocument.register_event_type('on_style_text')
 
+
 class UnformattedDocument(AbstractDocument):
-    '''A document having uniform style over all text.
+    """A document having uniform style over all text.
 
     Changes to the style of text within the document affects the entire
     document.  For convenience, the ``position`` parameters of the style
     methods may therefore be omitted.
-    '''
+    """
 
     def __init__(self, text=''):
         super(UnformattedDocument, self).__init__(text)
@@ -611,18 +619,19 @@ class UnformattedDocument(AbstractDocument):
         font_size = self.styles.get('font_size')
         bold = self.styles.get('bold', False)
         italic = self.styles.get('italic', False)
-        return font.load(font_name, font_size, 
-                         bold=bool(bold), italic=bool(italic), dpi=dpi) 
+        return font.load(font_name, font_size,
+                         bold=bool(bold), italic=bool(italic), dpi=dpi)
 
     def get_element_runs(self):
         return runlist.ConstRunIterator(len(self._text), None)
 
+
 class FormattedDocument(AbstractDocument):
-    '''Simple implementation of a document that maintains text formatting.
+    """Simple implementation of a document that maintains text formatting.
 
     Changes to text style are applied according to the description in
-    `AbstractDocument`.  All styles default to ``None``.
-    '''
+    :py:class:`~pyglet.text.document.AbstractDocument`.  All styles default to ``None``.
+    """
 
     def __init__(self, text=''):
         self._style_runs = {}
@@ -641,7 +650,7 @@ class FormattedDocument(AbstractDocument):
             return None
 
     def _set_style(self, start, end, attributes):
-        for attribute, value in list(attributes.items()):
+        for attribute, value in attributes.items():
             try:
                 runs = self._style_runs[attribute]
             except KeyError:
@@ -668,11 +677,11 @@ class FormattedDocument(AbstractDocument):
         super(FormattedDocument, self)._insert_text(start, text, attributes)
 
         len_text = len(text)
-        for runs in list(self._style_runs.values()):
+        for runs in self._style_runs.values():
             runs.insert(start, len_text)
 
         if attributes is not None:
-            for attribute, value in list(attributes.items()):
+            for attribute, value in attributes.items():
                 try:
                     runs = self._style_runs[attribute]
                 except KeyError:
@@ -683,9 +692,10 @@ class FormattedDocument(AbstractDocument):
 
     def _delete_text(self, start, end):
         super(FormattedDocument, self)._delete_text(start, end)
-        for runs in list(self._style_runs.values()):
+        for runs in self._style_runs.values():
             runs.delete(start, end)
 
+
 def _iter_elements(elements, length):
     last = 0
     for element in elements:
@@ -695,11 +705,13 @@ def _iter_elements(elements, length):
         last = p + 1
     yield last, length, None
 
+
 class _ElementIterator(runlist.RunIterator):
     def __init__(self, elements, length):
         self._run_list_iter = _iter_elements(elements, length)
         self.start, self.end, self.value = next(self)
 
+
 class _FontStyleRunsRangeIterator(object):
     # XXX subclass runlist
     def __init__(self, font_names, font_sizes, bolds, italics, dpi):
@@ -711,8 +723,8 @@ class _FontStyleRunsRangeIterator(object):
         from pyglet import font
         for start, end, styles in self.zip_iter.ranges(start, end):
             font_name, font_size, bold, italic = styles
-            ft = font.load(font_name, font_size, 
-                           bold=bool(bold), italic=bool(italic), 
+            ft = font.load(font_name, font_size,
+                           bold=bool(bold), italic=bool(italic),
                            dpi=self.dpi)
             yield start, end, ft
 
@@ -720,9 +732,10 @@ class _FontStyleRunsRangeIterator(object):
         from pyglet import font
         font_name, font_size, bold, italic = self.zip_iter[index]
         return font.load(font_name, font_size,
-                         bold=bool(bold), italic=bool(italic), 
+                         bold=bool(bold), italic=bool(italic),
                          dpi=self.dpi)
 
+
 class _NoStyleRangeIterator(object):
     # XXX subclass runlist
     def ranges(self, start, end):
@@ -730,4 +743,6 @@ class _NoStyleRangeIterator(object):
 
     def __getitem__(self, index):
         return None
+
+
 _no_style_range_iterator = _NoStyleRangeIterator()
