@@ -34,6 +34,8 @@
 
 '''
 '''
+from __future__ import division
+from builtins import chr
 
 __docformat__ = 'restructuredtext'
 __version__ = '$Id: $'
@@ -105,7 +107,8 @@ class Win32Window(BaseWindow):
     _exclusive_mouse = False
     _exclusive_mouse_focus = True
     _exclusive_mouse_screen = None
-    _exclusive_mouse_client = None
+    _exclusive_mouse_lpos = None
+    _exclusive_mouse_buttons = 0
     _mouse_platform_visible = True
 
     _ws_style = 0
@@ -170,7 +173,7 @@ class Win32Window(BaseWindow):
             white = _gdi32.GetStockObject(WHITE_BRUSH)
             black = _gdi32.GetStockObject(BLACK_BRUSH)
             self._window_class = WNDCLASS()
-            self._window_class.lpszClassName = 'GenericAppClass%d' % id(self)
+            self._window_class.lpszClassName = u'GenericAppClass%d' % id(self)
             self._window_class.lpfnWndProc = WNDPROC(
                 self._get_window_proc(self._event_handlers))
             self._window_class.style = CS_VREDRAW | CS_HREDRAW
@@ -184,7 +187,7 @@ class Win32Window(BaseWindow):
 
             self._view_window_class = WNDCLASS()
             self._view_window_class.lpszClassName = \
-                'GenericViewClass%d' % id(self)
+                u'GenericViewClass%d' % id(self)
             self._view_window_class.lpfnWndProc = WNDPROC(
                 self._get_window_proc(self._view_event_handlers))
             self._view_window_class.style = 0
@@ -200,7 +203,7 @@ class Win32Window(BaseWindow):
             self._hwnd = _user32.CreateWindowExW(
                 self._ex_ws_style,
                 self._window_class.lpszClassName,
-                '',
+                u'',
                 self._ws_style,
                 CW_USEDEFAULT,
                 CW_USEDEFAULT,
@@ -214,7 +217,7 @@ class Win32Window(BaseWindow):
             self._view_hwnd = _user32.CreateWindowExW(
                 0,
                 self._view_window_class.lpszClassName,
-                '',
+                u'',
                 WS_CHILD | WS_VISIBLE,
                 0, 0, 0, 0,
                 self._hwnd,
@@ -270,9 +273,9 @@ class Win32Window(BaseWindow):
 
         if self._visible:
             self.set_visible()
-            self.dispatch_event('on_expose')
             # Might need resize event if going from fullscreen to fullscreen
             self.dispatch_event('on_resize', self._width, self._height)
+            self.dispatch_event('on_expose')
 
     def _update_view_location(self, width, height):
         if self._fullscreen:
@@ -284,15 +287,22 @@ class Win32Window(BaseWindow):
             x, y, width, height, SWP_NOZORDER | SWP_NOOWNERZORDER)
 
     def close(self):
-        super(Win32Window, self).close()
         if not self._hwnd:
+            super(Win32Window, self).close()
             return
+
         _user32.DestroyWindow(self._hwnd)
         _user32.UnregisterClassW(self._window_class.lpszClassName, 0)
+        
+        self._window_class = None
+        self._view_window_class = None
+        self._view_event_handlers.clear()
+        self._event_handlers.clear()
         self.set_mouse_platform_visible(True)
         self._hwnd = None
         self._dc = None
         self._wgl_context = None
+        super(Win32Window, self).close()
 
     def _get_vsync(self):
         return self.context.get_vsync()
@@ -355,9 +365,9 @@ class Win32Window(BaseWindow):
             insertAfter = HWND_TOPMOST if self._fullscreen else HWND_TOP
             _user32.SetWindowPos(self._hwnd, insertAfter, 0, 0, 0, 0,
                 SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW)
-            self.dispatch_event('on_show')
-            self.activate()
             self.dispatch_event('on_resize', self._width, self._height)
+            self.activate()
+            self.dispatch_event('on_show')
         else:
             _user32.ShowWindow(self._hwnd, SW_HIDE)
             self.dispatch_event('on_hide')
@@ -415,19 +425,28 @@ class Win32Window(BaseWindow):
         # This is the point the mouse will be kept at while in exclusive
         # mode.
         self._exclusive_mouse_screen = p.x, p.y
-        self._exclusive_mouse_client = p.x - rect.left, p.y - rect.top
 
     def set_exclusive_mouse(self, exclusive=True):
         if self._exclusive_mouse == exclusive and \
            self._exclusive_mouse_focus == self._has_focus:
             return
 
-        if exclusive and self._has_focus:
-            # Move mouse to the center of the window.
-            self._reset_exclusive_mouse_screen()
-            x, y = self._exclusive_mouse_screen
-            self.set_mouse_position(x, y, absolute=True)
+        # Mouse: UsagePage = 1, Usage = 2
+        raw_mouse = RAWINPUTDEVICE(0x01, 0x02, 0, None)
+        if exclusive:
+            raw_mouse.dwFlags = RIDEV_NOLEGACY
+            raw_mouse.hwndTarget = self._view_hwnd
+        else:
+            raw_mouse.dwFlags = RIDEV_REMOVE
+            raw_mouse.hwndTarget = None
 
+        if not _user32.RegisterRawInputDevices(
+                byref(raw_mouse), 1, sizeof(RAWINPUTDEVICE)):
+            if exclusive:
+                raise WindowException("Cannot enter mouse exclusive mode.")
+
+        self._exclusive_mouse_buttons = 0
+        if exclusive and self._has_focus:
             # Clip to client area, to prevent large mouse movements taking
             # it outside the client area.
             rect = RECT()
@@ -435,13 +454,15 @@ class Win32Window(BaseWindow):
             _user32.MapWindowPoints(self._view_hwnd, HWND_DESKTOP,
                                     byref(rect), 2)
             _user32.ClipCursor(byref(rect))
+            # Release mouse capture in case is was acquired during mouse click
+            _user32.ReleaseCapture()
         else:
             # Release clip
             _user32.ClipCursor(None)
 
         self._exclusive_mouse = exclusive
         self._exclusive_mouse_focus = self._has_focus
-        self.set_mouse_platform_visible()
+        self.set_mouse_platform_visible(not exclusive)
 
     def set_mouse_position(self, x, y, absolute=False):
         if not absolute:
@@ -536,6 +557,7 @@ class Win32Window(BaseWindow):
                 byref(dataptr), None, 0)
             _user32.ReleaseDC(None, hdc)
 
+            image = image.get_image_data()
             data = image.get_data(format, pitch)
             memmove(dataptr, data, len(data))
 
@@ -556,13 +578,13 @@ class Win32Window(BaseWindow):
         image = best_image(_user32.GetSystemMetrics(SM_CXICON),
                            _user32.GetSystemMetrics(SM_CYICON))
         icon = get_icon(image)
-        _user32.SetClassLongW(self._hwnd, GCL_HICON, icon)
+        _user32.SetClassLongPtrW(self._hwnd, GCL_HICON, icon)
 
         # Set small icon
         image = best_image(_user32.GetSystemMetrics(SM_CXSMICON),
                            _user32.GetSystemMetrics(SM_CYSMICON))
         icon = get_icon(image)
-        _user32.SetClassLongW(self._hwnd, GCL_HICONSM, icon)
+        _user32.SetClassLongPtrW(self._hwnd, GCL_HICONSM, icon)
 
     # Private util
 
@@ -709,22 +731,90 @@ class Win32Window(BaseWindow):
         return 0
 
     @ViewEventHandler
-    @Win32EventHandler(WM_MOUSEMOVE)
-    def _event_mousemove(self, msg, wParam, lParam):
-        x, y = self._get_location(lParam)
-
-        if (x, y) == self._exclusive_mouse_client:
-            # Ignore the event caused by SetCursorPos
-            self._mouse_x = x
-            self._mouse_y = y
+    @Win32EventHandler(WM_INPUT)
+    def _event_raw_input(self, msg, wParam, lParam):
+        if not self._exclusive_mouse:
             return 0
 
-        y = self._height - y
+        hRawInput = cast(lParam, HRAWINPUT)
+        inp = RAWINPUT()
+        size = UINT(sizeof(inp))
+        _user32.GetRawInputData(hRawInput, RID_INPUT, byref(inp),
+                        byref(size), sizeof(RAWINPUTHEADER))
 
+        if inp.header.dwType == RIM_TYPEMOUSE:
+            rmouse = inp.data.mouse
+
+            if rmouse.usButtonFlags & RI_MOUSE_LEFT_BUTTON_DOWN:
+                self.dispatch_event('on_mouse_press', 0, 0, mouse.LEFT,
+                                    self._get_modifiers())
+                self._exclusive_mouse_buttons |= mouse.LEFT
+            if rmouse.usButtonFlags & RI_MOUSE_LEFT_BUTTON_UP:
+                self.dispatch_event('on_mouse_release', 0, 0, mouse.LEFT,
+                                    self._get_modifiers())
+                self._exclusive_mouse_buttons &= ~mouse.LEFT
+            if rmouse.usButtonFlags & RI_MOUSE_RIGHT_BUTTON_DOWN:
+                self.dispatch_event('on_mouse_press', 0, 0, mouse.RIGHT,
+                                    self._get_modifiers())
+                self._exclusive_mouse_buttons |= mouse.RIGHT
+            if rmouse.usButtonFlags & RI_MOUSE_RIGHT_BUTTON_UP:
+                self.dispatch_event('on_mouse_release', 0, 0, mouse.RIGHT,
+                                    self._get_modifiers())
+                self._exclusive_mouse_buttons &= ~mouse.RIGHT
+            if rmouse.usButtonFlags & RI_MOUSE_MIDDLE_BUTTON_DOWN:
+                self.dispatch_event('on_mouse_press', 0, 0, mouse.MIDDLE,
+                                    self._get_modifiers())
+                self._exclusive_mouse_buttons |= mouse.MIDDLE
+            if rmouse.usButtonFlags & RI_MOUSE_MIDDLE_BUTTON_UP:
+                self.dispatch_event('on_mouse_release', 0, 0, mouse.MIDDLE,
+                                    self._get_modifiers())
+                self._exclusive_mouse_buttons &= ~mouse.MIDDLE
+            if rmouse.usButtonFlags & RI_MOUSE_WHEEL:
+                delta = SHORT(rmouse.usButtonData).value
+                self.dispatch_event('on_mouse_scroll',
+                    0, 0, 0, delta / float(WHEEL_DELTA))
+
+            if rmouse.usFlags & 0x01 == MOUSE_MOVE_RELATIVE:
+                if rmouse.lLastX != 0 or rmouse.lLastY != 0:
+                    # Motion event
+                    # In relative motion, Y axis is positive for below.
+                    # We invert it for Pyglet so positive is motion up.
+                    if self._exclusive_mouse_buttons:
+                        self.dispatch_event('on_mouse_drag', 0, 0,
+                                            rmouse.lLastX, -rmouse.lLastY,
+                                            self._exclusive_mouse_buttons,
+                                            self._get_modifiers())
+                    else:
+                        self.dispatch_event('on_mouse_motion', 0, 0,
+                                            rmouse.lLastX, -rmouse.lLastY)
+            else:
+                if self._exclusive_mouse_lpos is None:
+                    self._exclusive_mouse_lpos = rmouse.lLastX, rmouse.lLastY
+                last_x, last_y = self._exclusive_mouse_lpos
+                rel_x = rmouse.lLastX - last_x
+                rel_y = rmouse.lLastY - last_y
+                if rel_x != 0 or rel_y != 0.0:
+                    # Motion event
+                    if self._exclusive_mouse_buttons:
+                        self.dispatch_event('on_mouse_drag', 0, 0,
+                                            rmouse.lLastX, -rmouse.lLastY,
+                                            self._exclusive_mouse_buttons,
+                                            self._get_modifiers())
+                    else:
+                        self.dispatch_event('on_mouse_motion', 0, 0,
+                                        rel_x, rel_y)
+                    self._exclusive_mouse_lpos = rmouse.lLastX, rmouse.lLastY
+
+        return 0
+
+    @ViewEventHandler
+    @Win32EventHandler(WM_MOUSEMOVE)
+    def _event_mousemove(self, msg, wParam, lParam):
         if self._exclusive_mouse and self._has_focus:
-            # Reset mouse position (so we don't hit the edge of the screen).
-            _x, _y = self._exclusive_mouse_screen
-            self.set_mouse_position(_x, _y, absolute=True)
+            return 0
+
+        x, y = self._get_location(lParam)
+        y = self._height - y
 
         dx = x - self._mouse_x
         dy = y - self._mouse_y
@@ -882,7 +972,6 @@ class Win32Window(BaseWindow):
         if not self._fullscreen:
             self._width, self._height = w, h
         self._update_view_location(self._width, self._height)
-        self._reset_exclusive_mouse_screen()
         self.switch_to()
         self.dispatch_event('on_resize', self._width, self._height)
         return 0
@@ -904,7 +993,6 @@ class Win32Window(BaseWindow):
     @Win32EventHandler(WM_MOVE)
     def _event_move(self, msg, wParam, lParam):
         x, y = self._get_location(lParam)
-        self._reset_exclusive_mouse_screen()
         self.dispatch_event('on_move', x, y)
         return 0
 
@@ -932,6 +1020,7 @@ class Win32Window(BaseWindow):
     def _event_setfocus(self, msg, wParam, lParam):
         self.dispatch_event('on_activate')
         self._has_focus = True
+
         self.set_exclusive_keyboard(self._exclusive_keyboard)
         self.set_exclusive_mouse(self._exclusive_mouse)
         return 0
@@ -940,8 +1029,18 @@ class Win32Window(BaseWindow):
     def _event_killfocus(self, msg, wParam, lParam):
         self.dispatch_event('on_deactivate')
         self._has_focus = False
-        self.set_exclusive_keyboard(self._exclusive_keyboard)
-        self.set_exclusive_mouse(self._exclusive_mouse)
+        exclusive_keyboard = self._exclusive_keyboard
+        exclusive_mouse = self._exclusive_mouse
+        # Disable both exclusive keyboard and mouse
+        self.set_exclusive_keyboard(False)
+        self.set_exclusive_mouse(False)
+
+        # But save desired state and note that we lost focus
+        # This will allow to reset the correct mode once we regain focus
+        self._exclusive_keyboard = exclusive_keyboard
+        self._exclusive_keyboard_focus = False
+        self._exclusive_mouse = exclusive_mouse
+        self._exclusive_mouse_focus = False
         return 0
 
     @Win32EventHandler(WM_GETMINMAXINFO)
